from typing import Any, Dict, List, Tuple, Union
import numpy as np


class WinMLModel:
    """
    Class representing a WinML model.

    Args:
        model_path (str): Path to the ONNX model file.
    """

    def __init__(self, model_path: str) -> None: ...

    def get_author(self) -> str: ...

    def get_name(self) -> str: ...

    def get_domain(self) -> str: ...

    def get_description(self) -> str: ...

    def get_input_names(self) -> List[str]: ...

    def get_output_names(self) -> List[str]: ...

    def get_input_details(self) -> List[Tuple[str, List[int], str]]: ...

    def get_output_details(self) -> List[Tuple[str, List[int], str]]: ...


class WinMLSession:
    """
    Class representing a WinML session for model inference.

    Args:
        model (WinMLModel): The WinML model to be used for inference.
        device_type (str): The device type to be used ('CPU' or 'GPU').

    Methods:
        set_input(name: str, value: WinMLValue) -> None:
            Set an input value for the session.

        evaluate() -> Dict[str, WinMLValue]:
            Run inference and return the output values.

        get_input_details() -> List[Tuple[str, List[int], str]]:
            Get details of model inputs.

        get_output_details() -> List[Tuple[str, List[int], str]]:
            Get details of model outputs.
    """

    def __init__(self, model: WinMLModel, device_type: str = 'CPU') -> None: ...

    def set_input(self, name: str, value: 'WinMLValue') -> None: ...

    def clear_input(self) -> None: ...

    def evaluate(self) -> Dict[str, 'WinMLValue']: ...

    def get_input_details(self) -> List[Tuple[str, List[int], str]]: ...

    def get_output_details(self) -> List[Tuple[str, List[int], str]]: ...


class WinMLValue:
    """
    Class representing a value to be used as input or output in WinML.

    Args:
        array (np.ndarray): The NumPy array representing the value.

    Methods:
        to_numpy() -> np.ndarray:
            Convert the value to a NumPy array.
    """

    def __init__(self, array: np.ndarray) -> None: ...

    def to_numpy(self) -> np.ndarray: ...


class DeviceType:
    """
    Enum representing the device type to be used for inference.
    """
    CPU: str
    GPU: str



import os
import re
import sys
import platform
import subprocess
from setuptools import setup, find_packages, Extension
from setuptools.command.build_ext import build_ext

class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=''):
        super().__init__(name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)

class CMakeBuild(build_ext):
    def run(self):
        try:
            subprocess.check_call(['cmake', '--version'])
        except OSError:
            raise RuntimeError("CMake must be installed to build the following extensions: " +
                               ", ".join(e.name for e in self.extensions))

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))
        cmake_args = [
            f'-DCMAKE_LIBRARY_OUTPUT_DIRECTORY={extdir}',
            f'-DPYTHON_EXECUTABLE={sys.executable}'
        ]

        cfg = 'Debug' if self.debug else 'Release'
        build_args = ['--config', cfg]

        if platform.system() == "Windows":
            cmake_args += [
                f'-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{cfg.upper()}={extdir}'
            ]
            if sys.maxsize > 2**32:
                cmake_args += ['-A', 'x64']
            build_args += ['--', '/m']
        else:
            build_args += ['--', '-j2']

        env = os.environ.copy()
        env['CXXFLAGS'] = '{} -DVERSION_INFO=\\"{}\\"'.format(
            env.get('CXXFLAGS', ''), self.distribution.get_version())
        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args, cwd=self.build_temp, env=env)
        subprocess.check_call(['cmake', '--build', '.'] + build_args, cwd=self.build_temp)

setup(
    name='win_onnx',
    version='0.1.1',
    author='Mosrat',
    author_email='whl2075928012@gmail.com',
    description='A lightweight library for binding WinML to Python, similar to ONNX Runtime.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/mosrat/win_onnx',
    packages=find_packages(where="packages"),
    package_dir={'': 'packages'},
    include_package_data=True,
    package_data={'win_onnx': ['winml.cp39-win_amd64.pyd']},
    # ext_modules=[CMakeExtension('win_onnx')],
    # cmdclass={'build_ext': CMakeBuild},
    python_requires=">=3.7",
    zip_safe=False,
    install_requires=[
        'numpy>=1.18.0',  # 添加对numpy的依赖
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)
